library(dplyr)
deviation <- function (x,...) {
	x <- factor(x,...) %>% C(sum)
	c <- contrasts(x)
	contrasts(x) <- c / nrow(c)
	x
}
data <- read.csv('data.csv') %>% mutate(
	gender_listener = deviation(gender_listener,levels=c('M','F')),
	education_listener = factor(education_listener,levels=c('primary school','vmbo','mbo','havo','vwo','hbo','university')) %>% C(helmert),
	gender_speaker = deviation(gender_speaker,levels=c('M','F')),
	speaker = factor(speaker),
	origin_speaker = deviation(origin_speaker,levels=c(unique(origin_speaker) %>% .[. != 'Goes'],'Goes'))
)
edges <- read.csv('edges.csv')
grid  <- read.csv('grid.csv')

# Visualization
dviz <- mutate(data, bin = interaction(cut(answer_lat,10),cut(answer_lon,10))) %>%
	group_by(origin_speaker, bin) %>%
	summarize(N = n(), lat = median(answer_lat), lon = median(answer_lon))
dviz$origin_speaker <- ifelse(dviz$origin_speaker == 'Alphen aan den Rijn','Alphen a.d. Rijn',as.character(dviz$origin_speaker))
answers <- select(data,correct_lat,correct_lon,origin_speaker) %>%
	unique() %>%
	mutate(origin_speaker = ifelse(origin_speaker == 'Alphen aan den Rijn','Alphen a.d. Rijn',as.character(origin_speaker)))
library(ggplot2)
pviz <- ggplot(dviz,aes(x=lon,y=lat,size=N)) +
	scale_size_area(expression(italic('N'))) + coord_map('stereographic') +
	geom_polygon(aes(X,Y,group=interaction(L1,L2,L3)),edges,fill='gray',color='black',inherit.aes=FALSE) +
	geom_point() +
	xlab('Answer (longitude)') + ylab('Answer (latitude)') +
	facet_wrap(~origin_speaker,ncol=4) + theme_bw() +
	geom_point(aes(x=correct_lon,y=correct_lat),answers,color='black',fill='white',shape=24,size=2,inherit.aes=FALSE)
tiff('pviz.tif',width=8-1/4-2,height=11-3/4-2,units='in',res=300)
print(pviz)
dev.off()

# GAM analysis
data$gender_listener_contrast <- as.vector(model.matrix(~data$gender_listener)[,2])
for (i in 1:6) {
	data[[paste0('education_listener_contrast',i)]] <- as.vector(model.matrix(~data$education_listener)[,i+1])
}
formula.km <- list(
	accent_recognition_error ~ gender_listener + education_listener + gender_speaker +
		s(geographical_knowledge_listener,k=10) + s(age_listener,k=10) + s(distance_own_region,k=10) + s(origin_listener_lat,origin_listener_lon,bs='sos',k=10) +
		s(speaker,bs='re') +
		s(speaker,gender_listener_contrast,bs='re') +
		s(speaker,education_listener_contrast1,bs='re') +
		s(speaker,education_listener_contrast2,bs='re') +
		s(speaker,education_listener_contrast3,bs='re') +
		s(speaker,education_listener_contrast4,bs='re') +
		s(speaker,education_listener_contrast5,bs='re') +
		s(speaker,education_listener_contrast6,bs='re') +
		s(speaker,geographical_knowledge_listener,bs='fs',k=10,m=1) +
		s(age_listener,speaker,bs='fs',k=10,m=1) +
		s(distance_own_region,speaker,bs='fs',k=10,m=1) +
		s(origin_listener_lat,origin_listener_lon,speaker,bs='fs',xt=list(bs='sos'),k=10),
	~s(distance_own_region,k=10) + s(origin_listener_lat,origin_listener_lon,bs='sos',k=10) +
		s(speaker,bs='re') +
		s(distance_own_region,speaker,bs='fs',k=10,m=1) +
		s(origin_listener_lat,origin_listener_lon,speaker,bs='fs',xt=list(bs='sos'),k=10)
	)
model.km <- gam(formula.km,data=data,family=gammals,control=list(trace=TRUE))
save('model.km',file='model.km.RData',compress='xz',compression_level=-9)
sink('summary.km.txt')
print(summary(model.km))
sink()

# add interaction to test if it helps (discussion section)
formula.km[[1]] <- update(formula.km[[1]], . ~ . + ti(geographical_knowledge_listener,age_listener,bs=c('ts','ts'),k=c(10,10))) #ts has a penalized null space and hence makes REML comparison possible (gammals is always fitted with REML)
model.km.ti <- gam(formula.km,data=data,family=gammals,control=list(trace=TRUE))
save('model.km.ti',file='model.km.ti.RData',compress='xz',compression_level=-9)

# Visualization of the GAM results
gen.seq <- function (x,len=100) seq(min(data[[x]],na.rm=TRUE),max(data[[x]],na.rm=TRUE),length.out=len)
make.newdata <- function (...) {
	todo <- list(...)
	nd <- data[1,]
	nd <- nd[rep(1,length(todo[[1]])),]
	for (x in names(todo)) {
		nd[[x]] <- todo[[x]]
	}
	nd
}
make.lp <- function (model,newdata,terms) {
	lp <- predict(model,newdata=newdata,type='lpmatrix')
	del.lss  <- attr(lp,'lpi')[[2]]
	del.term <- which(sapply(colnames(lp),function (x) !any(grepl(terms,x))))
	del.rand <- which(grepl('speaker',colnames(lp)))
	lp[,c(del.lss,del.term,del.rand)] <- 0
	lp
}
make.df <- function (model,newdata,lp) within(newdata,{
	y <- as.vector(lp %*% coef(model))
	se <- sqrt(diag(lp %*% vcov(model) %*% t(lp)))
	cimin <- y - qnorm(.025,lower.tail=FALSE)*se
	cimax <- y + qnorm(.025,lower.tail=FALSE)*se
	sig <- sign(cimin) == sign(cimax)
})

newdata.geographical_knowledge_listener <- plyr::adply(gen.seq('geographical_knowledge_listener'),1,function (x) make.newdata(geographical_knowledge_listener=x),.id=NULL)
lp.geographical_knowledge_listener <- make.lp(model.km,newdata.geographical_knowledge_listener,'geographical_knowledge_listener')
df.geographical_knowledge_listener <- make.df(model.km,newdata.geographical_knowledge_listener,lp.geographical_knowledge_listener)

newdata.age_listener <- plyr::adply(gen.seq('age_listener'),1,function (x) make.newdata(age_listener=x),.id=NULL)
lp.age_listener <- make.lp(model.km,newdata.age_listener,'age_listener')
df.age_listener <- make.df(model.km,newdata.age_listener,lp.age_listener)

newdata.distance_own_region <- plyr::adply(gen.seq('distance_own_region'),1,function (x) make.newdata(distance_own_region=x),.id=NULL)
lp.distance_own_region <- make.lp(model.km,newdata.distance_own_region,'distance_own_region')
df.distance_own_region <- make.df(model.km,newdata.distance_own_region,lp.distance_own_region)

newdata.origin_listener <- make.newdata(origin_listener_lat=grid$lat,origin_listener_lon=grid$lon)
lp.origin_listener <- make.lp(model.km,newdata.origin_listener,'origin_listener')
df.origin_listener <- make.df(model.km,newdata.origin_listener,lp.origin_listener)

plot.geographical_knowledge_listener <- ggplot(df.geographical_knowledge_listener,aes(x=geographical_knowledge_listener,y=y)) + geom_ribbon(aes(ymin=cimin,ymax=cimax),alpha=.1) + geom_line() + xlab('Geographical knowledge (10-point scale)') + ylab('Accent Recognition Error (log km)') + theme_bw()
plot.age_listener <- ggplot(df.age_listener,aes(x=age_listener,y=y)) + geom_ribbon(aes(ymin=cimin,ymax=cimax),alpha=.1) + geom_line() + xlab('Listener age (years)') + ylab('Accent Recognition Error (log km)') + theme_bw()
plot.distance_own_region <- ggplot(df.distance_own_region,aes(x=distance_own_region,y=y)) + geom_ribbon(aes(ymin=cimin,ymax=cimax),alpha=.1) + geom_line() + xlab('Distance to own region (km)') + ylab('Accent Recognition Error (log km)') + theme_bw()
plot.origin_listener <- ggplot(df.origin_listener,aes(x=origin_listener_lon,y=origin_listener_lat,fill=y,color=y)) + geom_tile() + scale_fill_gradient(low='white',high='black') + scale_color_gradient(low='white',high='black') + geom_polygon(aes(X,Y,group=interaction(L1,L2,L3)),fill='white',color='black',alpha=0,edges,inherit.aes=FALSE) + coord_map('stereographic') + xlab('Listener origin (longitude)') + ylab('Listener origin (latitude)') + theme_bw() + labs(color='Accent Recognition Error (log km)',fill='Accent Recognition Error (log km)')

for (x in c('plot.geographical_knowledge_listener','plot.age_listener','plot.distance_own_region','plot.origin_listener')) {
	tiff(paste0(x,'.tif'),width=8-1/4-2,height=5,units='in',res=300)
	print(get(x))
	dev.off()
}
